==> dclib-0.3.18.tar.bz2: dclib-0.3.18/dclib/cfilehasher.h <==
/***************************************************************************
        cfilehasher.h  -  Calculate the TTH root and leaves for a file
                             -------------------
    begin                : Fri May 16 2008
    copyright            : (C) 2008 by Edward Sheldrake
    email                : ejs1920@yahoo.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CFILEHASHER_H
#define CFILEHASHER_H

/**
 * @author Edward Sheldrake
 */

#include <dclib/dcos.h>
#include <dclib/core/cobject.h>
#include <dclib/core/cthread.h>
#include <dclib/core/cstring.h>
#include <dclib/core/cfile.h>
#include <dclib/core/cbytearray.h>
#include <dclib/core/clist.h>

/* The DC++ sources use different types than dclib: int64_t vs ulonglong */
#include <stdint.h>

/** */
typedef enum eFileHasherStatus {
	efhsNotStarted=0,
	efhsReady,
	efhsWorking,
	efhsFinished,
	efhsError
} eFileHasherStatus;

class DLL_EXPORT CHashedSegment : public CObject {

public:
	/** Constructor */
	CHashedSegment() {
		start = 0;
		size  = 0;
	};
	/** Destructor */
	virtual ~CHashedSegment() { };
	
	/** start position */
	ulonglong start;
	/** chunk size */
	ulonglong size;
	/** expected hash, base32 encoded */
	CString expected;
	/** actual hash, base32 encoded */
	CString actual;
};

class DLL_EXPORT CTreeVerificationReport : public CObject {

public:
	/** Constructor */
	CTreeVerificationReport() {
		filesize = 0;
		allgood  = FALSE;
		segments = new CList<CHashedSegment>;
	};
	
	/** Destructor */
	virtual ~CTreeVerificationReport() {
		if ( segments != 0 )
		{
			delete segments;
			segments = 0;
		}
	};
	
	/** Returns a nice string of the data that you can print out. */
	CString ToString() const;
	
	/** full name of file */
	CString filename;
	/** actual TTH root */
	CString tthRoot;
	/** file size */
	ulonglong filesize;
	/** true if all hashes matched */
	bool allgood;
	/** the list of file segments and their hashes */
	CList<CHashedSegment> * segments;
};
class DLL_EXPORT CFileHasher : public CObject {

public:
	/** Constructor */
	CFileHasher( const CString filename, CByteArray * workmem = 0 );
	/** Destructor */
	virtual ~CFileHasher();
	
	/** 
	 * Calculate the hash.
	 * Sets status to the given value when finished.
	 */
	void ComputeHash( const eFileHasherStatus doneStatus = efhsFinished, const CString progressPrefix = CString() );
	/** Stop the hashing */
	void StopHashing();
	/** */
	bool IsStop() const { return m_bStop; };
	/** Get status */
	eFileHasherStatus GetStatus() const { return status; };
	/** Get progress in the form "bytes done/bytes total"*/
	CString GetProgress() const { return m_sProgress; };
	/** Get the filesize */
	ulonglong GetFileSize() const { return filesize; };
	/** Get the TTH root as a base32 encoded cstring */
	CString GetHashRoot();
	/** Returns a newly allocated copy of the hash root data */
	CByteArray * GetHashRootRaw();
	/** Returns a newly allocated copy of the hash leaf data */
	CByteArray * GetLeafData();
	/** Returns a pointer to the hash root data, so be careful with it.*/
	CByteArray * HashRootDirect() { return m_pRootData; };
	/** Returns a pointer to the hash leaf data, so be careful with it.*/
	CByteArray * HashLeavesDirect() { return m_pLeafData; };
	
	/** Returns true if the base32 encoded TTH root matches the TTH leaf data */
	static bool ValidateHashLeaves( const CString tth, const CByteArray * leaves, const ulonglong filesize );
	/** The same, but with the raw TTH root not the base32 encoded string */
	static bool ValidateHashLeaves( const CByteArray * root, const CByteArray * leaves, const ulonglong filesize );
	/** */
	static int64_t GetBlockSize( const unsigned long leavesSize, const int64_t filesize );
	/** Get the root TTH of a bytearray as raw */
	static CByteArray * HashByteArray( CByteArray * data, unsigned long size );

protected:
	/** Set status */
	void SetStatus( const eFileHasherStatus );
	/** Set the progress */
	void SetProgress( const CString s );

	/** The file to hash */
	CFile file;
	/** The size of the file */
	ulonglong filesize;	
	/** Memory area to read file data to */
	CByteArray * m_pWorkMem;
	/** True if we allocated our own */
	bool usingOwnMem;
	/** Store the final TTH root data */
	CByteArray * m_pRootData;
	/** Store the final leaf data */
	CByteArray * m_pLeafData;

private:
	/** Used to lock/unlock private things */
	CMutex m_Mutex;
	/** Progress of the operation */
	CString m_sProgress;
	/** Set to true to stop the hashing */
	bool m_bStop;
	/** Status */
	eFileHasherStatus status;
};

class DLL_EXPORT CFileHasherThread : public CFileHasher, public CThread {

public:
	/** Constructor */
	CFileHasherThread ( const CString filename, CByteArray * workmem = 0 ) : CFileHasher( filename, workmem ) { };
	/** Destructor */
	virtual ~CFileHasherThread() { };
	
	/** Thread callback function */
	virtual void Thread( CObject * Object );
};

class DLL_EXPORT CFileTreeVerifier: public CFileHasher, public CThread {

public:
	/** Constructor */
	CFileTreeVerifier( const CString filename, CByteArray * leaves, CByteArray * workmem = 0 );
	/** Destructor */
	virtual ~CFileTreeVerifier();
	
	/**
	 * Gets the results of the verification.
	 * If finished, Returns the pointer to our report and sets m_pReport=0
	 * so we won't delete it when we're deleted.
	 * Whoever gets it must delete it when done with it.
	 * Otherwise returns 0 (and returns 0 the second time if you call it twice).
	 */
	CTreeVerificationReport * GetReport();
	
	/** Thread callback function */
	virtual void Thread( CObject * Object );

private:
	/** The hash leaves we are going to compare the file with */
	CByteArray * m_pLeaves;
	/** The output of the verification */
	CTreeVerificationReport * m_pReport;
};

#endif // CFILEHASHER_H

==> dclib-0.3.21.tar.bz2: dclib-0.3.21/dclib/cfilehasher.h <==
/***************************************************************************
        cfilehasher.h  -  Calculate the TTH root and leaves for a file
                             -------------------
    begin                : Fri May 16 2008
    copyright            : (C) 2008 by Edward Sheldrake
    email                : ejs1920@yahoo.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CFILEHASHER_H
#define CFILEHASHER_H

/**
 * @author Edward Sheldrake
 */

#include <dclib/dcos.h>
#include <dclib/core/cthread.h>
#include <dclib/core/cstring.h>
#include <dclib/core/cfile.h>

#include <dclib/core/clist.h>

/* The DC++ sources use different types than dclib: int64_t vs ulonglong */
#include <stdint.h>

class CByteArray;

/** */
typedef enum eFileHasherStatus {
	efhsNotStarted=0,
	efhsReady,
	efhsWorking,
	efhsFinished,
	efhsError
} eFileHasherStatus;

class DLL_EXPORT CHashedSegment {

public:
	/** Constructor */
	CHashedSegment() {
		start = 0;
		size  = 0;
	};
	/** Destructor */
	~CHashedSegment() { };
	
	/** start position */
	ulonglong start;
	/** chunk size */
	ulonglong size;
	/** expected hash, base32 encoded */
	CString expected;
	/** actual hash, base32 encoded */
	CString actual;
};

class DLL_EXPORT CTreeVerificationReport {

public:
	/** Constructor */
	CTreeVerificationReport() {
		filesize = 0;
		allgood  = false;
		segments = new CList<CHashedSegment>;
	};
	
	/** Copy constructor */
	CTreeVerificationReport( const CTreeVerificationReport & other );
	
	/** Destructor */
	~CTreeVerificationReport() {
		if ( segments != 0 )
		{
			delete segments;
			segments = 0;
		}
	};
	
	/** Returns a nice string of the data that you can print out. */
	CString ToString() const;
	
	/** full name of file */
	CString filename;
	/** actual TTH root */
	CString tthRoot;
	/** file size */
	ulonglong filesize;
	/** true if all hashes matched */
	bool allgood;
	/** the list of file segments and their hashes */
	CList<CHashedSegment> * segments;
};
class DLL_EXPORT CFileHasher {

public:
	/** Constructor */
	CFileHasher( const CString filename, CByteArray * workmem = 0 );
	/** Destructor */
	virtual ~CFileHasher();
	
	/** 
	 * Calculate the hash.
	 * Sets status to the given value when finished.
	 */
	void ComputeHash( const eFileHasherStatus doneStatus = efhsFinished, const CString progressPrefix = CString() );
	/** Stop the hashing */
	void StopHashing();
	/** */
	bool IsStop() const { return m_bStop; };
	/** Get status */
	eFileHasherStatus GetStatus() const { return status; };
	/** Get progress in the form "bytes done/bytes total"*/
	CString GetProgress() const { return m_sProgress; };
	/** Get the filesize */
	ulonglong GetFileSize() const { return filesize; };
	/** Get the TTH root as a base32 encoded cstring */
	CString GetHashRoot();
	/** Returns a newly allocated copy of the hash root data */
	CByteArray * GetHashRootRaw();
	/** Returns a newly allocated copy of the hash leaf data */
	CByteArray * GetLeafData();
	/** Returns a pointer to the hash root data, so be careful with it.*/
	CByteArray * HashRootDirect() { return m_pRootData; };
	/** Returns a pointer to the hash leaf data, so be careful with it.*/
	CByteArray * HashLeavesDirect() { return m_pLeafData; };
	
	/** Returns true if the base32 encoded TTH root matches the TTH leaf data */
	static bool ValidateHashLeaves( const CString tth, CByteArray * leaves, const ulonglong filesize );
	/** The same, but with the raw TTH root not the base32 encoded string */
	static bool ValidateHashLeaves( CByteArray * root, CByteArray * leaves, const ulonglong filesize );
	/** */
	static int64_t GetBlockSize( const unsigned long leavesSize, const int64_t filesize );
	/** Get the root TTH of a bytearray as raw */
	static CByteArray * HashByteArray( CByteArray * data, unsigned long size );
	/** Get the size in bytes of the unencoded TTH value */
	static unsigned long HashSize();

protected:
	/** Set status */
	void SetStatus( const eFileHasherStatus );
	/** Set the progress */
	void SetProgress( const CString s );

	/** The file to hash */
	CFile file;
	/** The size of the file */
	ulonglong filesize;	
	/** Memory area to read file data to */
	CByteArray * m_pWorkMem;
	/** True if we allocated our own */
	bool usingOwnMem;
	/** Store the final TTH root data */
	CByteArray * m_pRootData;
	/** Store the final leaf data */
	CByteArray * m_pLeafData;

private:
	/** Used to lock/unlock private things */
	CMutex m_Mutex;
	/** Progress of the operation */
	CString m_sProgress;
	/** Set to true to stop the hashing */
	bool m_bStop;
	/** Status */
	eFileHasherStatus status;
};

class DLL_EXPORT CFileHasherThread : public CFileHasher, public CThread {

public:
	/** Constructor */
	CFileHasherThread ( const CString filename, CByteArray * workmem = 0 ) : CFileHasher( filename, workmem ) { };
	/** Destructor */
	virtual ~CFileHasherThread() { };
	
	/** Thread callback function */
	virtual void Thread();
};

class DLL_EXPORT CFileTreeVerifier: public CFileHasher, public CThread {

public:
	/** Constructor */
	CFileTreeVerifier( const CString filename, CByteArray * leaves, CByteArray * workmem = 0 );
	/** Destructor */
	virtual ~CFileTreeVerifier();
	
	/**
	 * Gets the results of the verification.
	 * If finished, Returns the pointer to our report and sets m_pReport=0
	 * so we won't delete it when we're deleted.
	 * Whoever gets it must delete it when done with it.
	 * Otherwise returns 0 (and returns 0 the second time if you call it twice).
	 */
	CTreeVerificationReport * GetReport();
	
	/** Thread callback function */
	virtual void Thread();

private:
	/** The hash leaves we are going to compare the file with */
	CByteArray * m_pLeaves;
	/** The output of the verification */
	CTreeVerificationReport * m_pReport;
};

#endif // CFILEHASHER_H
